@@ -5,7 +5,7 @@
 #' OmicsPLS includes the O2PLS fit, the GO2PLS fit, cross-validation tools and some misc functions.
 #' 
 #' @author
-#' Said el Bouhaddani (\email{s.elbouhaddani@@umcutrecht.nl}, Twitter: @@selbouhaddani),
+#' Said el Bouhaddani (\email{s.elbouhaddani@@umcutrecht.nl}, LinkedIn: @@in/selbouhaddani),
 #' Zhujie Gu, 
 #' Szymon Kielbasa,
 #' Geurt Jongbloed,
@@ -44,8 +44,8 @@
 #' 
 #' @section Obtaining results:
 #' After fitting an O2PLS model, by running e.g. \code{fit = o2m(X,Y,n,nx,ny)}, the results can be visualised.
-#' Use \code{\link{plot}(fit,...)} to plot the desired loadings with/without ggplot2.
-#' Use \code{\link{summary}(fit,...)} to see the relative explained variances in the joint/orthogonal parts.
+#' Use \code{\link[OmicsPLS:plot.o2m]{plot}(fit,...)} to plot the desired loadings with/without ggplot2.
+#' Use \code{\link[OmicsPLS:summary.o2m]{summary}(fit,...)} to see the relative explained variances in the joint/orthogonal parts.
 #' Also plotting the joint scores \code{fit$Tt, fit$U} and orthogonal scores \code{fit$T_Yosc, fit$U_Xosc} are of help.
 #' 
 #' @section Cross-validating: 
@@ -63,10 +63,10 @@
 #' @section S3 methods:
 #' There are S3 methods implemented for a fit obtained with \code{o2m}, i.e. \code{fit <- o2m(X,Y,n,nx,ny)}
 #' \itemize{
-#'   \item{} Use plot(fit) to plot the loadings, see above.
-#'   \item{} Use \code{\link{loadings}(fit)} to extract a matrix with loading values
-#'   \item{} Use \code{\link{scores}(fit)} to extract the scores
-#'   \item{} Use \code{\link{print}} and \code{\link{summary}} to print and summarize the fit object
+#'   \item{} Use \code{\link[OmicsPLS:plot.o2m]{plot}(fit)} to plot the loadings, see above.
+#'   \item{} Use \code{\link[OmicsPLS:loadings]{loadings}(fit)} to extract a matrix with loading values
+#'   \item{} Use \code{\link[OmicsPLS:scores]{scores}(fit)} to extract the scores
+#'   \item{} Use \code{\link[OmicsPLS:print.o2m]{print}} and \code{\link[OmicsPLS:summary.o2m]{summary}} to print and summarize the fit object
 #' }
 #' 
 #' @section Imputation:
@@ -74,7 +74,7 @@
 #' There are many sophisticated approaches available, such as MICE and MissForest, and no one approach is the best for all situations.
 #' To still allow users to quickly impute missing values in their data matrix, 
 #' the \code{\link{impute_matrix}} function is implemented. 
-#' It relies on the \code{\link{softImpute}} function+package and imputes based on the singular value decomposition.
+#' It relies on the \code{\link[softImpute:softImpute]{softImpute}} function+package and imputes based on the singular value decomposition.
 #' 
 #' @section Misc:
 #' Also some handy tools are available
@@ -100,14 +100,15 @@
 #' \emph{O2-PLS, a two-block (X-Y) latent variable regression (LVR) method with an integral OSC filter.} 
 #' Journal of Chemometrics, 17(1), 53-64. \doi{10.1002/cem.775}
 #' 
-#' @docType package
-#' @name OmicsPLS
+#' 
+#' @aliases OmicsPLS-package
 #' @keywords OmicsPLS
 #' @import parallel ggplot2 tibble magrittr softImpute
 #' @importFrom graphics abline
-#' @importFrom stats cov sd
+#' @importFrom stats cov sd predict 
 #' @importFrom dplyr mutate
-NULL
+#' @importFrom withr with_seed
+"_PACKAGE"
 
 #' Check if matrices satisfy input conditions
 #'
@@ -140,7 +141,7 @@ input_checker <- function(X, Y = NULL) {
 #' @param X A matrix with missing values in some entries.
 #' @param ... Further arguments for \code{softimpute}.
 #' @return An imputed version of matrix \eqn{X}
-#' @details This function is based on the \code{\link{softImpute}} function in its eponymous package.
+#' @details This function is based on the \code{\link[softImpute:softImpute]{softImpute}} function in its eponymous package.
 #' @examples
 #' X <- matrix(rnorm(20*100),20)
 #' Xmis <- X
@@ -259,7 +260,8 @@ rmsep <- function(Xtst, Ytst, fit, combi = FALSE) {
   
   input_checker(Xtst, Ytst)
   
-  Yhat <- Xtst %*% fit$W. %*% fit$B_T %*% t(fit$C.)
+  #Yhat <- Xtst %*% fit$W. %*% fit$B_T %*% t(fit$C.)
+  Yhat <- predict(fit, Xtst, "X")
   # Xhat = Ytst%*%fit$C.%*%fit$B_U%*%t(fit$W.)
   
   return(mean(c(sqrt(mse(Yhat, Ytst)))))  #,sqrt(mse(Xhat,Xtst)))))
@@ -277,10 +279,12 @@ rmsep <- function(Xtst, Ytst, fit, combi = FALSE) {
 #' @param func Function to fit the O2PLS model with. Only \code{\link{o2m}} and \code{\link{o2m_stripped}} are supported.
 #' @param app_err Logical. Deprecated. Should the apparent error also be computed? 
 #' @param kcv Integer. The value of \eqn{k}, i.e. the number of folds. Choose \eqn{N} for LOO-CV.
+#' @param seed Integer. A random seed to make the analysis reproducible.
 #' @details Note that this function can be easily parallelized (on Windows e.g. with the \code{parallel} package.).
 #' @return List with two numeric vectors:
 #' \item{CVerr}{Contains the k-fold CV estimated RMSEP}
 #' \item{Fiterr}{Contains the apparent error}
+#' @importFrom withr with_seed
 #' @details The parameters \code{a}, \code{a2} and \code{b2} can be integers or vectors of integers. A for loop is used to loop over all combinations.
 #' The resulting output is a list, which is more easy to interpret if you use \code{array(unlist(output_of_loocv$CVerr))} as in the example below.
 #' The array wil have varying \code{a} along the first dimension and \code{a2} and \code{b2} along the second and third respectively.
@@ -288,7 +292,7 @@ rmsep <- function(Xtst, Ytst, fit, combi = FALSE) {
 #' @export
 loocv <- function(X, Y, a = 1:2, a2 = 1, b2 = 1, fitted_model = NULL, func = o2m, app_err = F, kcv,
                   stripped = TRUE, p_thresh = 3000, 
-                  q_thresh = p_thresh, tol = 1e-10, max_iterations = 100)
+                  q_thresh = p_thresh, tol = 1e-10, max_iterations = 100, seed = "off")
 {
   app_err = F
   fitted_model = NULL
@@ -296,6 +300,8 @@ loocv <- function(X, Y, a = 1:2, a2 = 1, b2 = 1, fitted_model = NULL, func = o2m
   X = as.matrix(X)
   Y = as.matrix(Y)
   input_checker(X, Y)
+  if(!is.numeric(seed) & (seed != "off")) stop("seed has to be set to 'off' or an integer")
+  if(!is.numeric(seed)) with_seed <- function(ii, code) code
   # if (!is.null(fitted_model)) {
   #   app_err <- F
   #   message("apparent error calculated with provided fit","\n")
@@ -319,7 +325,7 @@ loocv <- function(X, Y, a = 1:2, a2 = 1, b2 = 1, fitted_model = NULL, func = o2m
       for (j3 in b2) {
         k <- k + 1
         err <- NA * 1:kcv
-        folds <- sample(N)
+        folds <- with_seed(seed + k, sample(N))
         # loop through number of folds
         for (i in 1:kcv) {
           ii <- which(blocks==i)
@@ -365,7 +371,7 @@ loocv <- function(X, Y, a = 1:2, a2 = 1, b2 = 1, fitted_model = NULL, func = o2m
 #' @param a2 Vector of integers. Contains the numbers of orthogonal components in \eqn{X}.
 #' @param b2 Vector of integers. Contains the numbers of orthogonal components in \eqn{Y}.
 #' @param func Function to fit the O2PLS model with. Only \code{\link{o2m}} and \code{\link{o2m_stripped}} are supported.
-#' @param parall Integer. Should a parallel cluster be set up using package \code{parallel} (Windows)? Best is to leave it to \code{FALSE}.
+#' @param parall Integer. Should a parallel cluster be set up using package \code{\link[parallel:parallel]{parallel}} (Windows)? Best is to leave it to \code{FALSE}.
 #' @param cl Object of class '\code{cluster}'. If parall is \code{TRUE} and \code{cl} is not \code{NULL}, calculations are parallelized over workers in cl.
 #' @details The use of this function is to calculate the R2 of the joint part, while varying the number of orthogonal components. Adding more joint components will increase the R2!
 #'
@@ -395,6 +401,7 @@ adjR2 <- function(X, Y, a = 1:2, a2 = 1, b2 = 1, func = o2m, parall = F, cl = NU
     S_apply <- parSapply
     cl <- makeCluster(rep("localhost", detectCores()), type = "SOCK")
     clusterExport(cl = cl, varlist = c("ssq", "o2m_stripped", "adjR2"))
+    on.exit(stopCluster(cl))
   }
   if (parall & !is.null(cl)) {
     stopifnot(inherits(cl,'cluster'))
@@ -424,9 +431,7 @@ adjR2 <- function(X, Y, a = 1:2, a2 = 1, b2 = 1, func = o2m, parall = F, cl = NU
       return(c(adjR2X = NA, adjR2Y = NA))
     }
   })
-  if (parall & cl_was_null == TRUE) {
-    stopCluster(cl)
-  }
+  
   return(outp)
 }
 
@@ -468,9 +473,10 @@ rmsep_combi <- function(Xtst, Ytst, fit)
   
   input_checker(Xtst, Ytst)
   
-  Yhat <- (Xtst - Xtst %*% fit$W_Yosc %*% t(fit$W_Yosc)) %*% fit$W. %*% fit$B_T %*% t(fit$C.)
-  Xhat <- (Ytst - Ytst %*% fit$C_Xosc %*% t(fit$C_Xosc)) %*% fit$C. %*% fit$B_U %*% t(fit$W.)
-  
+  #Yhat <- (Xtst - Xtst %*% fit$W_Yosc %*% t(fit$W_Yosc)) %*% fit$W. %*% fit$B_T %*% t(fit$C.)
+  #Xhat <- (Ytst - Ytst %*% fit$C_Xosc %*% t(fit$C_Xosc)) %*% fit$C. %*% fit$B_U %*% t(fit$W.)
+  Yhat <- predict(fit, Xtst, "X")
+  Xhat <- predict(fit, Ytst, "Y")
   return(sqrt(mse(Yhat, Ytst)) + sqrt(mse(Xhat, Xtst)))
 }
 
@@ -488,7 +494,7 @@ rmsep_combi <- function(Xtst, Ytst, fit)
 #' @export
 loocv_combi <- function(X, Y, a = 1:2, a2 = 1, b2 = 1, fitted_model = NULL, func = o2m, app_err = F, kcv,
                         stripped = TRUE, p_thresh = 3000, 
-                        q_thresh = p_thresh, tol = 1e-10, max_iterations = 100)
+                        q_thresh = p_thresh, tol = 1e-10, max_iterations = 100, seed = "off")
 {
   app_err = F
   fitted_model = NULL
@@ -497,6 +503,8 @@ loocv_combi <- function(X, Y, a = 1:2, a2 = 1, b2 = 1, fitted_model = NULL, func
   X = as.matrix(X)
   Y = as.matrix(Y)
   input_checker(X, Y)
+  if(!is.numeric(seed) & (seed != "off")) stop("seed has to be set to 'off' or an integer")
+  if(!is.numeric(seed)) with_seed <- function(ii, code) code
   # if (!is.null(fitted_model)) {
   #   if(inherits(fitted_model,c("o2m","pre.o2m"))){stop("fitted_model should be of class 'o2m' or NULL","\n")}
   #   app_err <- F
@@ -523,7 +531,7 @@ loocv_combi <- function(X, Y, a = 1:2, a2 = 1, b2 = 1, fitted_model = NULL, func
       for (j3 in b2) {
         k <- k + 1
         err <- NA * 1:kcv
-        folds <- sample(N)
+        folds <- with_seed(seed + k, sample(N))
         # loop through number of folds
         for (i in 1:kcv) {
           ii <- which(blocks==i)
@@ -589,8 +597,8 @@ print.o2m <- function (x, ...) {
   n = x$flags$n #ncol(x$W.)
   nx = x$flags$nx #ifelse(vnorm(x$P_Yosc.)[1] == 0, 0, ncol(x$P_Yosc.))
   ny = x$flags$ny #ifelse(vnorm(x$P_Xosc.)[1] == 0, 0, ncol(x$P_Xosc.))
-  if(x$flags$method == "SO2PLS") cat("SO2PLS fit \n")
-  else if(x$flags$method == "GO2PLS") cat("GO2PLS fit \n")
+  if(x$flags$method == "SO2PLS") {cat("SO2PLS fit \n")}
+  else if(x$flags$method == "GO2PLS") {cat("GO2PLS fit \n")}
   else{
     if(x$flags$stripped) cat("O2PLS fit: Stripped \n") 
     else if(x$flags$highd) cat("O2PLS fit: High dimensional \n") 
@@ -626,13 +634,13 @@ print.pre.o2m <- function (x, ...) {
 #' @param loading_name character string. One of the following: 'Xjoint', 'Yjoint', 'gr_Xjoint', 'gr_Yjoint', 'Xorth' or 'Yorth'.
 #' @param i Integer. First component to be plotted.
 #' @param j NULL (default) or Integer. Second component to be plotted.
-#' @param use_ggplot2 Logical. Default is \code{TRUE}. If \code{FALSE}, the usual plot device will be used.
+#' @param use_ggplot2 Deprecated. Logical. Default is \code{TRUE}. If \code{FALSE}, the usual plot device will be used.
 #' @param label Character, either 'number' or 'colnames'. The first option prints numbers, the second prints the colnames
 #' @param ... Further arguments to \code{geom_text}, such as size, col, alpha, etc.
 #' 
 #' @return If \code{use_ggplot2} is \code{TRUE} a ggplot2 object. Else NULL.
 #' 
-#' @seealso \code{\link{summary.o2m}}
+#' @seealso \code{\link[OmicsPLS:summary.o2m]{summary}}
 #' 
 #' @export
 plot.o2m <- function (x, loading_name = c("Xjoint", "Yjoint", "gr_Xjoint", "gr_Yjoint", "Xorth", "Yorth"), i = 1, j = NULL, use_ggplot2=TRUE, label = c("number", "colnames"), ...)
@@ -640,6 +648,7 @@ plot.o2m <- function (x, loading_name = c("Xjoint", "Yjoint", "gr_Xjoint", "gr_Y
   stopifnot(i == round(i), is.logical(use_ggplot2))
   
   fit <- list()
+  if(length(label) > 2) stop("label should be a single string, 'number' or 'colnames'")
   loading_name = match.arg(loading_name)
   if((loading_name %in% c("gr_Xjoint", "gr_Yjoint")) & x$flags$method != "GO2PLS") stop("Loading plots at group level only available in GO2PLS")
   
@@ -695,7 +704,7 @@ plot.o2m <- function (x, loading_name = c("Xjoint", "Yjoint", "gr_Xjoint", "gr_Y
 #' @examples
 #' summary(o2m(scale(-2:2),scale(-2:2*4),1,0,0))
 #' 
-#' @seealso \code{\link{plot.o2m}}
+#' @seealso \code{\link[OmicsPLS:plot.o2m]{plot}}
 #' 
 #' @export
 summary.o2m <- function(object, digits = 3, ...) {
@@ -804,7 +813,7 @@ print.summary.o2m <- function(x, ...){
 #' @examples
 #' loadings(o2m(scale(-2:2),scale(-2:2*4),1,0,0))
 #' 
-#' @seealso \code{\link{scores.o2m}}
+#' @seealso \code{\link[OmicsPLS:scores.o2m]{scores}}
 #' 
 #' @rdname loadings
 #' @export
@@ -853,7 +862,7 @@ loadings.o2m <- function(x, loading_name = c("Xjoint", "Yjoint", "gr_Xjoint", "g
 #' @examples
 #' scores(o2m(scale(-2:2),scale(-2:2*4),1,0,0))
 #' 
-#' @seealso \code{\link{loadings.o2m}}
+#' @seealso \code{\link[OmicsPLS:loadings.o2m]{loadings}}
 #' 
 #' @rdname scores
 #' @export
@@ -1144,7 +1153,7 @@ orth_vec <- function(x, W){
       n <- nrow(W)
       I <- diag(1, n)
       #print(W)
-      x_sub <- (I - W %*% solve(t(W)%*%W) %*% t(W)) %*% x_sub
+      x_sub <- x_sub - W %*% solve(t(W)%*%W) %*% (t(W) %*% x_sub)
     }
     
     x[rowi] <- x_sub
